@@ -45,7 +45,7 @@
 #define MAX_TGROUPS 16
 #endif
 
-#define MAX_THREADS_PER_GROUP LONGBITS
+#define MAX_THREADS_PER_GROUP __WORDSIZE
 
 /* threads enabled, max_threads defaults to long bits for 1 tgroup or 4 times
  * long bits if more tgroups are enabled.
@@ -250,6 +250,12 @@
 #define COOKIE_DELIM_DATE       '|'
 #endif
 
+// Max number of acl() sample fetch recursive evaluations, to avoid deep tree
+// loops.
+#ifndef ACL_MAX_RECURSE
+#define ACL_MAX_RECURSE 1000
+#endif
+
 #define CONN_RETRIES    3
 
 #define	CHK_CONNTIME    2000
@@ -271,6 +277,13 @@
 // X-Original-To header default
 #define DEF_XORIGINALTO_HDR	"X-Original-To"
 
+/* Max number of events that may be processed at once by
+ * an event_hdl API consumer to prevent thread contention.
+ */
+#ifndef EVENT_HDL_MAX_AT_ONCE
+#define EVENT_HDL_MAX_AT_ONCE 100
+#endif
+
 /* Default connections limit.
  *
  * A system limit can be enforced at build time in order to avoid using haproxy
@@ -289,24 +302,6 @@
 #define DEFAULT_MAXCONN 100
 #endif
 
-/* Default file descriptor limit.
- *
- * DEFAULT_MAXFD explicitly reduces the hard RLIMIT_NOFILE, which is used by the
- * process as the base value to calculate the default global.maxsock, if
- * global.maxconn, global.rlimit_memmax are not defined. This is useful in the
- * case, when hard nofile limit has been bumped to fs.nr_open (kernel max),
- * which is extremely large on many modern distros. So, we will also finish with
- * an extremely large default global.maxsock. The only way to override
- * DEFAULT_MAXFD, if defined, is to set fd_hard_limit in the config global
- * section. If DEFAULT_MAXFD is not set, a reasonable maximum of 1048576 will be
- * used as the default value, which almost guarantees that a process will
- * correctly start in any situation and will be not killed then by watchdog,
- * when it will loop over the allocated fdtab.
-*/
-#ifndef DEFAULT_MAXFD
-#define DEFAULT_MAXFD 1048576
-#endif
-
 /* Define a maxconn which will be used in the master process once it re-exec to
  * the MODE_MWORKER_WAIT and won't change when SYSTEM_MAXCONN is set.
  *
@@ -466,6 +461,23 @@
 #define CONFIG_HAP_POOL_CLUSTER_SIZE 8
 #endif
 
+/* number of bits to encode the per-pool buckets for large setups */
+#ifndef CONFIG_HAP_POOL_BUCKETS_BITS
+# if defined(USE_THREAD) && MAX_THREADS >= 512
+#  define CONFIG_HAP_POOL_BUCKETS_BITS 6
+# elif defined(USE_THREAD) && MAX_THREADS >= 128
+#  define CONFIG_HAP_POOL_BUCKETS_BITS 5
+# elif defined(USE_THREAD) && MAX_THREADS >= 16
+#  define CONFIG_HAP_POOL_BUCKETS_BITS 4
+# elif defined(USE_THREAD)
+#  define CONFIG_HAP_POOL_BUCKETS_BITS 3
+# else
+#  define CONFIG_HAP_POOL_BUCKETS_BITS 0
+# endif
+#endif
+
+#define CONFIG_HAP_POOL_BUCKETS (1UL << (CONFIG_HAP_POOL_BUCKETS_BITS))
+
 /* Number of samples used to compute the times reported in stats. A power of
  * two is highly recommended, and this value multiplied by the largest response
  * time must not overflow and unsigned int. See freq_ctr.h for more information.
@@ -509,12 +521,13 @@
 /* system sysfs directory */
 #define NUMA_DETECT_SYSTEM_SYSFS_PATH "/sys/devices/system"
 
-/* Elements used by memory profiling. This determines the number of buckets to
- * store stats.
- */
-#ifndef MEMPROF_HASH_BITS
-# define MEMPROF_HASH_BITS 10
+/* Number of cache trees */
+#ifndef CACHE_TREE_NUM
+# if defined(USE_THREAD)
+#  define CACHE_TREE_NUM 8
+# else
+#  define CACHE_TREE_NUM 1
+# endif
 #endif
-#define MEMPROF_HASH_BUCKETS (1U << MEMPROF_HASH_BITS)
 
 #endif /* _HAPROXY_DEFAULTS_H */
